@@ -1,6 +1,7 @@
 #define TMB_LIB_INIT R_init_foieGras
 #include <TMB.hpp>
-#include "TMB/sub/ssm.hpp"
+#include "TMB/sub/rw.hpp"
+#include "TMB/sub/crw.hpp"
 #include "TMB/sub/mpm.hpp"
 #include "TMB/sub/joint_mpm.hpp"
 
@@ -8,8 +9,10 @@ template<class Type>
 Type objective_function<Type>::operator() ()
 {
   DATA_STRING(model_name);
-  if (model_name == "ssm") {
-    return ssm(this);
+  if (model_name == "rw") {
+    return rw(this);
+  } else if (model_name == "crw") {
+    return crw(this);
   } else if (model_name == "mpm") {
     return mpm(this);
   } else if (model_name == "jmpm") {
